@@ -19,11 +19,9 @@
 #ifndef BROWSERENTRYCONFIG_H
 #define BROWSERENTRYCONFIG_H
 
-#include "Variant.h"
-#include <QtCore/QObject>
-#include <QtCore/QSet>
-#include <QtCore/QString>
-#include <QtCore/QStringList>
+#include <QObject>
+#include <QSet>
+#include <QString>
 
 class Entry;
 
