@@ -63,6 +63,7 @@ public:
         GlobalAutoTypeRetypeTime,
         FaviconDownloadTimeout,
         UpdateCheckMessageShown,
+        DefaultDatabaseFileName,
 
         LastDatabases,
         LastKeyFiles,
@@ -85,6 +86,7 @@ public:
         GUI_HideUsernames,
         GUI_HidePasswords,
         GUI_AdvancedSettings,
+        GUI_ColorPasswords,
         GUI_MonospaceNotes,
         GUI_ApplicationTheme,
         GUI_CompactMode,
