@@ -19,12 +19,8 @@
 #ifndef KEEPASSX_ENTRY_H
 #define KEEPASSX_ENTRY_H
 
-#include <QImage>
 #include <QMap>
-#include <QPixmap>
 #include <QPointer>
-#include <QSet>
-#include <QUrl>
 #include <QUuid>
 
 #include "core/AutoTypeAssociations.h"
@@ -36,6 +32,8 @@
 
 class Database;
 class Group;
+class PasswordHealth;
+
 namespace Totp
 {
     struct Settings;
@@ -60,19 +58,22 @@ struct EntryData
     QString foregroundColor;
     QString backgroundColor;
     QString overrideUrl;
-    QString tags;
+    QStringList tags;
     bool autoTypeEnabled;
     int autoTypeObfuscation;
     QString defaultAutoTypeSequence;
     TimeInfo timeInfo;
     QSharedPointer<Totp::Settings> totpSettings;
+    QSharedPointer<PasswordHealth> passwordHealth;
+    bool excludeFromReports;
+    QUuid previousParentGroupUuid;
 
     bool operator==(const EntryData& other) const;
     bool operator!=(const EntryData& other) const;
     bool equals(const EntryData& other, CompareItemOptions options) const;
 };
 
-class Entry : public QObject
+class Entry : public ModifiableObject
 {
     Q_OBJECT
 
@@ -81,24 +82,24 @@ public:
     ~Entry();
     const QUuid& uuid() const;
     const QString uuidToHex() const;
-    QImage icon() const;
-    QPixmap iconPixmap(IconSize size = IconSize::Default) const;
     int iconNumber() const;
     const QUuid& iconUuid() const;
     QString foregroundColor() const;
     QString backgroundColor() const;
     QString overrideUrl() const;
     QString tags() const;
+    QStringList tagList() const;
     const TimeInfo& timeInfo() const;
     bool autoTypeEnabled() const;
     int autoTypeObfuscation() const;
     QString defaultAutoTypeSequence() const;
     QString effectiveAutoTypeSequence() const;
-    QString effectiveNewAutoTypeSequence() const;
+    QList<QString> autoTypeSequences(const QString& pattern = {}) const;
     AutoTypeAssociations* autoTypeAssociations();
     const AutoTypeAssociations* autoTypeAssociations() const;
     QString title() const;
     QString url() const;
+    QStringList getAllUrls() const;
     QString webUrl() const;
     QString displayUrl() const;
     QString username() const;
@@ -108,10 +109,19 @@ public:
     QString totp() const;
     QString totpSettingsString() const;
     QSharedPointer<Totp::Settings> totpSettings() const;
+    Group* previousParentGroup();
+    const Group* previousParentGroup() const;
+    QUuid previousParentGroupUuid() const;
     int size() const;
+    QString path() const;
+    const QSharedPointer<PasswordHealth> passwordHealth();
+    const QSharedPointer<PasswordHealth> passwordHealth() const;
+    bool excludeFromReports() const;
+    void setExcludeFromReports(bool state);
 
     bool hasTotp() const;
     bool isExpired() const;
+    bool willExpireInDays(int days) const;
     bool isRecycled() const;
     bool isAttributeReference(const QString& key) const;
     bool isAttributeReferenceOf(const QString& key, const QUuid& uuid) const;
@@ -145,6 +155,11 @@ public:
     void setExpires(const bool& value);
     void setExpiryTime(const QDateTime& dateTime);
     void setTotp(QSharedPointer<Totp::Settings> settings);
+    void setPreviousParentGroup(const Group* group);
+    void setPreviousParentGroupUuid(const QUuid& uuid);
+
+    void addTag(const QString& tag);
+    void removeTag(const QString& tag);
 
     QList<Entry*> historyItems();
     const QList<Entry*>& historyItems() const;
@@ -241,7 +256,7 @@ public:
 
     Group* group();
     const Group* group() const;
-    void setGroup(Group* group);
+    void setGroup(Group* group, bool trackPrevious = true);
     const Database* database() const;
     Database* database();
 
@@ -253,7 +268,6 @@ signals:
      * Emitted when a default attribute has been changed.
      */
     void entryDataChanged(Entry* entry);
-    void entryModified();
 
 private slots:
     void emitDataChanged();
