@@ -20,6 +20,9 @@
 
 #include "core/Global.h"
 
+#include <QRegularExpression>
+#include <QUuid>
+
 const QString EntryAttributes::TitleKey = "Title";
 const QString EntryAttributes::UserNameKey = "UserName";
 const QString EntryAttributes::PasswordKey = "Password";
@@ -27,7 +30,6 @@ const QString EntryAttributes::URLKey = "URL";
 const QString EntryAttributes::NotesKey = "Notes";
 const QStringList EntryAttributes::DefaultAttributes(QStringList()
                                                      << TitleKey << UserNameKey << PasswordKey << URLKey << NotesKey);
-
 const QString EntryAttributes::WantedFieldGroupName = "WantedField";
 const QString EntryAttributes::SearchInGroupName = "SearchIn";
 const QString EntryAttributes::SearchTextGroupName = "SearchText";
@@ -35,7 +37,7 @@ const QString EntryAttributes::SearchTextGroupName = "SearchText";
 const QString EntryAttributes::RememberCmdExecAttr = "_EXEC_CMD";
 
 EntryAttributes::EntryAttributes(QObject* parent)
-    : QObject(parent)
+    : ModifiableObject(parent)
 {
     clear();
 }
@@ -104,7 +106,7 @@ bool EntryAttributes::isReference(const QString& key) const
 
 void EntryAttributes::set(const QString& key, const QString& value, bool protect)
 {
-    bool emitModified = false;
+    bool shouldEmitModified = false;
 
     bool addAttribute = !m_attributes.contains(key);
     bool changeValue = !addAttribute && (m_attributes.value(key) != value);
@@ -116,27 +118,27 @@ void EntryAttributes::set(const QString& key, const QString& value, bool protect
 
     if (addAttribute || changeValue) {
         m_attributes.insert(key, value);
-        emitModified = true;
+        shouldEmitModified = true;
     }
 
     if (protect) {
         if (!m_protectedAttributes.contains(key)) {
-            emitModified = true;
+            shouldEmitModified = true;
         }
         m_protectedAttributes.insert(key);
     } else if (m_protectedAttributes.remove(key)) {
-        emitModified = true;
+        shouldEmitModified = true;
     }
 
-    if (emitModified) {
-        emit entryAttributesModified();
+    if (shouldEmitModified) {
+        emitModified();
     }
 
     if (defaultAttribute && changeValue) {
         emit defaultKeyModified();
     } else if (addAttribute) {
         emit added(key);
-    } else if (emitModified) {
+    } else if (shouldEmitModified) {
         emit customKeyModified(key);
     }
 }
@@ -155,7 +157,7 @@ void EntryAttributes::remove(const QString& key)
     m_protectedAttributes.remove(key);
 
     emit removed(key);
-    emit entryAttributesModified();
+    emitModified();
 }
 
 void EntryAttributes::rename(const QString& oldKey, const QString& newKey)
@@ -185,7 +187,7 @@ void EntryAttributes::rename(const QString& oldKey, const QString& newKey)
         m_protectedAttributes.insert(newKey);
     }
 
-    emit entryAttributesModified();
+    emitModified();
     emit renamed(oldKey, newKey);
 }
 
@@ -217,7 +219,7 @@ void EntryAttributes::copyCustomKeysFrom(const EntryAttributes* other)
     }
 
     emit reset();
-    emit entryAttributesModified();
+    emitModified();
 }
 
 bool EntryAttributes::areCustomKeysDifferent(const EntryAttributes* other)
@@ -250,7 +252,7 @@ void EntryAttributes::copyDataFrom(const EntryAttributes* other)
         m_protectedAttributes = other->m_protectedAttributes;
 
         emit reset();
-        emit entryAttributesModified();
+        emitModified();
     }
 }
 
@@ -303,7 +305,7 @@ void EntryAttributes::clear()
     }
 
     emit reset();
-    emit entryAttributesModified();
+    emitModified();
 }
 
 int EntryAttributes::attributesSize() const
