@@ -20,7 +20,6 @@
 #define KEEPASSX_ENTRYSEARCHER_H
 
 #include <QRegularExpression>
-#include <QString>
 
 class Group;
 class Entry;
@@ -39,7 +38,9 @@ public:
         AttributeKV,
         Attachment,
         AttributeValue,
-        Group
+        Group,
+        Tag,
+        Is
     };
 
     struct SearchTerm
@@ -70,7 +71,6 @@ private:
 
     bool m_caseSensitive;
     bool m_skipProtected;
-    QRegularExpression m_termParser;
     QList<SearchTerm> m_searchTerms;
 
     friend class TestEntrySearcher;
