@@ -23,8 +23,8 @@
 #include <QTimer>
 
 #include "gui/DialogyWidget.h"
-#include "keys/CompositeKey.h"
 
+class CompositeKey;
 class Database;
 class QFile;
 
@@ -45,6 +45,8 @@ public:
     void clearForms();
     void enterKey(const QString& pw, const QString& keyFile);
     QSharedPointer<Database> database();
+    void resetQuickUnlock();
+    bool unlockingDatabase();
 
 signals:
     void dialogFinished(bool accepted);
@@ -53,6 +55,9 @@ protected:
     void showEvent(QShowEvent* event) override;
     void hideEvent(QHideEvent* event) override;
     QSharedPointer<CompositeKey> buildDatabaseKey();
+    void setUserInteractionLock(bool state);
+    // Quick Unlock helper functions
+    bool isOnQuickUnlockScreen();
 
     const QScopedPointer<Ui::DatabaseOpenWidget> m_ui;
     QSharedPointer<Database> m_db;
@@ -65,7 +70,6 @@ protected slots:
 
 private slots:
     void browseKeyFile();
-    void clearKeyFileText();
     void pollHardwareKey();
     void hardwareKeyResponse(bool found);
     void openHardwareKeyHelp();
@@ -73,6 +77,8 @@ private slots:
 
 private:
     bool m_pollingHardwareKey = false;
+    bool m_blockQuickUnlock = false;
+    bool m_unlockingDatabase = false;
     QTimer m_hideTimer;
 
     Q_DISABLE_COPY(DatabaseOpenWidget)
