@@ -22,8 +22,6 @@
 #include <QPushButton>
 #include <QScrollArea>
 
-#include "core/Resources.h"
-
 EditWidget::EditWidget(QWidget* parent)
     : DialogyWidget(parent)
     , m_ui(new Ui::EditWidget())
@@ -102,7 +100,7 @@ void EditWidget::setPageHidden(QWidget* widget, bool hidden)
         m_ui->categoryList->setCategoryHidden(index, hidden);
     }
 
-    if (hidden && index == m_ui->stackedWidget->currentIndex()) {
+    if (index == m_ui->stackedWidget->currentIndex()) {
         int newIndex = m_ui->stackedWidget->currentIndex() - 1;
         if (newIndex < 0) {
             newIndex = m_ui->stackedWidget->count() - 1;
