@@ -20,10 +20,7 @@
 #define KEEPASSX_SETUPTOTPDIALOG_H
 
 #include "core/Database.h"
-#include "core/Entry.h"
 #include "gui/DatabaseWidget.h"
-#include <QDialog>
-#include <QScopedPointer>
 
 namespace Ui
 {
