@@ -22,8 +22,6 @@
 #include "gui/DialogyWidget.h"
 
 #include <QPointer>
-#include <QScopedPointer>
-#include <QSharedPointer>
 
 class Database;
 class DatabaseSettingsWidgetGeneral;
@@ -32,6 +30,7 @@ class DatabaseSettingsWidgetDatabaseKey;
 #ifdef WITH_XC_BROWSER
 class DatabaseSettingsWidgetBrowser;
 #endif
+class DatabaseSettingsWidgetMaintenance;
 class QTabWidget;
 
 namespace Ui
@@ -90,6 +89,7 @@ private:
 #ifdef WITH_XC_BROWSER
     QPointer<DatabaseSettingsWidgetBrowser> m_browserWidget;
 #endif
+    QPointer<DatabaseSettingsWidgetMaintenance> m_maintenanceWidget;
 
     class ExtraPage;
     QList<ExtraPage> m_extraPages;
