@@ -38,7 +38,9 @@ public:
     void setModel(QAbstractItemModel* model) override;
     Entry* currentEntry();
     void setCurrentEntry(Entry* entry);
+    QList<Entry*> selectedEntries();
     Entry* entryFromIndex(const QModelIndex& index);
+    QModelIndex indexFromEntry(Entry* entry);
     int currentEntryIndex();
     bool inSearchMode();
     bool isSorted();
@@ -58,12 +60,10 @@ signals:
 protected:
     void keyPressEvent(QKeyEvent* event) override;
     void focusInEvent(QFocusEvent* event) override;
-    void focusOutEvent(QFocusEvent* event) override;
     void showEvent(QShowEvent* event) override;
 
 private slots:
     void emitEntryActivated(const QModelIndex& index);
-    void emitEntrySelectionChanged();
     void showHeaderMenu(const QPoint& position);
     void toggleColumnVisibility(QAction* action);
     void fitColumnsToWindow();
