@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2020 KeePassXC Team <team@keepassxc.org>
+ * Copyright (C) 2021 KeePassXC Team <team@keepassxc.org>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,8 @@
 #define KEEPASSXC_OSUTILSBASE_H
 
 #include <QObject>
-#include <QPointer>
+
+class QWindow;
 
 /**
  * Abstract base class for generic OS-specific functionality
@@ -55,7 +56,20 @@ public:
      */
     virtual bool isCapslockEnabled() = 0;
 
+    virtual void registerNativeEventFilter() = 0;
+
+    virtual bool registerGlobalShortcut(const QString& name,
+                                        Qt::Key key,
+                                        Qt::KeyboardModifiers modifiers,
+                                        QString* error = nullptr) = 0;
+    virtual bool unregisterGlobalShortcut(const QString& name) = 0;
+
+    virtual bool canPreventScreenCapture() const = 0;
+    virtual bool setPreventScreenCapture(QWindow* window, bool allow) const;
+
 signals:
+    void globalShortcutTriggered(const QString& name, const QString& search = {});
+
     /**
      * Indicates platform UI theme change (light mode to dark mode).
      */
