@@ -60,7 +60,7 @@
 - (void) didDeactivateApplicationObserver:(NSNotification*) notification
 {
     NSDictionary* userInfo = notification.userInfo;
-    NSRunningApplication* app = userInfo[NSWorkspaceApplicationKey];
+    NSRunningApplication* app = [userInfo objectForKey:NSWorkspaceApplicationKey];
 
     if (app.processIdentifier != [self ownProcessId]) {
         self.lastActiveApplication = app;
@@ -139,6 +139,7 @@
 //
 - (bool) isStatusBarDark
 {
+#if __clang_major__ >= 9 && MAC_OS_X_VERSION_MIN_REQUIRED >= 101000
     if (@available(macOS 10.17, *)) {
         // This is an ugly hack, but I couldn't find a way to access QTrayIcon's NSStatusItem.
         NSStatusItem* dummy = [[NSStatusBar systemStatusBar] statusItemWithLength:0];
@@ -146,6 +147,7 @@
         [[NSStatusBar systemStatusBar] removeStatusItem:dummy];
         return [appearance containsString:@"dark"];
     }
+#endif
 
     return [self isDarkMode];
 }
@@ -166,9 +168,13 @@
 //
 - (bool) enableAccessibility
 {
+#if MAC_OS_X_VERSION_MIN_REQUIRED >= 1090
     // Request accessibility permissions for Auto-Type type on behalf of the user
     NSDictionary* opts = @{static_cast<id>(kAXTrustedCheckOptionPrompt): @YES};
     return AXIsProcessTrustedWithOptions(static_cast<CFDictionaryRef>(opts));
+#else
+    return YES;
+#endif
 }
 
 //
@@ -176,6 +182,7 @@
 //
 - (bool) enableScreenRecording
 {
+#if __clang_major__ >= 9 && MAC_OS_X_VERSION_MIN_REQUIRED >= 1080
     if (@available(macOS 10.15, *)) {
         // Request screen recording permission on macOS 10.15+
         // This is necessary to get the current window title
@@ -193,6 +200,7 @@
             return NO;
         }
     }
+#endif
     return YES;
 }
 
@@ -205,6 +213,11 @@
     }
 }
 
+- (void) setWindowSecurity:(NSWindow*) window state:(bool) state
+{
+    [window setSharingType: state ? NSWindowSharingNone : NSWindowSharingReadOnly];
+}
+
 @end
 
 //
@@ -279,3 +292,9 @@ void AppKit::toggleForegroundApp(bool foreground)
 {
     [static_cast<id>(self) toggleForegroundApp:foreground];
 }
+
+void AppKit::setWindowSecurity(QWindow* window, bool state)
+{
+    auto view = reinterpret_cast<NSView*>(window->winId());
+    [static_cast<id>(self) setWindowSecurity:view.window state:state];
+}
