@@ -17,14 +17,9 @@
 
 #include "SettingsPageKeeShare.h"
 
-#include "core/Database.h"
-#include "core/Group.h"
-#include "core/Resources.h"
 #include "gui/DatabaseTabWidget.h"
-#include "gui/MessageWidget.h"
-#include "keeshare/KeeShare.h"
+#include "gui/Icons.h"
 #include "keeshare/SettingsWidgetKeeShare.h"
-#include <QApplication>
 #include <QObject>
 
 SettingsPageKeeShare::SettingsPageKeeShare(DatabaseTabWidget* tabWidget)
@@ -34,12 +29,12 @@ SettingsPageKeeShare::SettingsPageKeeShare(DatabaseTabWidget* tabWidget)
 
 QString SettingsPageKeeShare::name()
 {
-    return QApplication::tr("KeeShare");
+    return "KeeShare";
 }
 
 QIcon SettingsPageKeeShare::icon()
 {
-    return Resources::instance()->icon("preferences-system-network-sharing");
+    return icons()->icon("preferences-system-network-sharing");
 }
 
 QWidget* SettingsPageKeeShare::createWidget()
