@@ -19,7 +19,6 @@
 #ifndef KEEPASSX_KEY_H
 #define KEEPASSX_KEY_H
 
-#include <QByteArray>
 #include <QUuid>
 
 class Key
@@ -30,6 +29,9 @@ public:
     Q_DISABLE_COPY(Key);
     virtual ~Key() = default;
     virtual QByteArray rawKey() const = 0;
+    virtual void setRawKey(const QByteArray& data) = 0;
+    virtual QByteArray serialize() const = 0;
+    virtual void deserialize(const QByteArray& data) = 0;
     inline virtual QUuid uuid() const
     {
         return m_uuid;
