@@ -20,11 +20,11 @@
 #define KEEPASSXC_SSHAGENT_H
 
 #include <QHash>
-#include <QList>
-#include <QtCore>
 
-#include "crypto/ssh/OpenSSHKey.h"
-#include "sshagent/KeeAgentSettings.h"
+#include "OpenSSHKey.h"
+
+class KeeAgentSettings;
+class Database;
 
 class SSHAgent : public QObject
 {
@@ -37,11 +37,16 @@ public:
     bool isEnabled() const;
     void setEnabled(bool enabled);
     QString socketPath(bool allowOverride = true) const;
+    QString securityKeyProvider(bool allowOverride = true) const;
     QString authSockOverride() const;
+    QString securityKeyProviderOverride() const;
     void setAuthSockOverride(QString& authSockOverride);
+    void setSecurityKeyProviderOverride(QString& securityKeyProviderOverride);
 #ifdef Q_OS_WIN
     bool useOpenSSH() const;
+    bool usePageant() const;
     void setUseOpenSSH(bool useOpenSSH);
+    void setUsePageant(bool usePageant);
 #endif
 
     const QString errorString() const;
@@ -72,8 +77,10 @@ private:
 
     const quint8 SSH_AGENT_CONSTRAIN_LIFETIME = 1;
     const quint8 SSH_AGENT_CONSTRAIN_CONFIRM = 2;
+    const quint8 SSH_AGENT_CONSTRAIN_EXTENSION = 255;
 
     bool sendMessage(const QByteArray& in, QByteArray& out);
+    bool sendMessageOpenSSH(const QByteArray& in, QByteArray& out);
 #ifdef Q_OS_WIN
     bool sendMessagePageant(const QByteArray& in, QByteArray& out);
 
