@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2019 KeePassXC Team <team@keepassxc.org>
+ *  Copyright (C) 2021 KeePassXC Team <team@keepassxc.org>
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,7 +22,6 @@
 
 #include "browser/BrowserAction.h"
 #include "browser/BrowserService.h"
-#include "core/Group.h"
 
 class TestBrowser : public QObject
 {
@@ -38,24 +37,26 @@ private slots:
     void testGetBase64FromKey();
     void testIncrementNonce();
 
-    void testBaseDomain();
+    void testTopLevelDomain();
+    void testIsIpAddress();
     void testSortPriority();
     void testSortPriority_data();
     void testSearchEntries();
+    void testSearchEntriesByPath();
+    void testSearchEntriesByUUID();
     void testSearchEntriesWithPort();
     void testSearchEntriesWithAdditionalURLs();
     void testInvalidEntries();
     void testSubdomainsAndPaths();
-    void testSortEntries();
     void testValidURLs();
     void testBestMatchingCredentials();
     void testBestMatchingWithAdditionalURLs();
 
 private:
     QList<Entry*> createEntries(QStringList& urls, Group* root) const;
+    void compareEntriesByPath(QSharedPointer<Database> db, QList<Entry*> entries, QString path);
 
     QScopedPointer<BrowserAction> m_browserAction;
     QPointer<BrowserService> m_browserService;
 };
-
 #endif // KEEPASSXC_TESTBROWSER_H
