@@ -20,18 +20,10 @@
 
 #include "TestKeePass2Format.h"
 
-class TestKdbx4 : public TestKeePass2Format
+class TestKdbx4Argon2 : public TestKeePass2Format
 {
     Q_OBJECT
 
-private slots:
-    void testFormat400();
-    void testFormat400Upgrade();
-    void testFormat400Upgrade_data();
-    void testUpgradeMasterKeyIntegrity();
-    void testUpgradeMasterKeyIntegrity_data();
-    void testCustomData();
-
 protected:
     void initTestCaseImpl() override;
 
@@ -40,19 +32,37 @@ protected:
     readXml(const QString& path, bool strictMode, bool& hasError, QString& errorString) override;
     void writeXml(QBuffer* buf, Database* db, bool& hasError, QString& errorString) override;
 
-    void readKdbx(const QString& path,
-                  QSharedPointer<const CompositeKey> key,
-                  QSharedPointer<Database> db,
-                  bool& hasError,
-                  QString& errorString) override;
     void readKdbx(QIODevice* device,
                   QSharedPointer<const CompositeKey> key,
                   QSharedPointer<Database> db,
                   bool& hasError,
                   QString& errorString) override;
     void writeKdbx(QIODevice* device, Database* db, bool& hasError, QString& errorString) override;
+};
 
-    QSharedPointer<Kdf> fastKdf(QSharedPointer<Kdf> kdf);
+class TestKdbx4AesKdf : public TestKdbx4Argon2
+{
+    Q_OBJECT
+
+protected:
+    void initTestCaseImpl() override;
+};
+
+/**
+ * KDF-independent KDBX4 format tests.
+ */
+class TestKdbx4Format : public QObject
+{
+    Q_OBJECT
+
+private slots:
+    void testFormat400();
+    void testFormat400Upgrade();
+    void testFormat400Upgrade_data();
+    void testFormat410Upgrade();
+    void testUpgradeMasterKeyIntegrity();
+    void testUpgradeMasterKeyIntegrity_data();
+    void testCustomData();
 };
 
 #endif // KEEPASSXC_TEST_KDBX4_H
